@@ -15,8 +15,6 @@
  * @since 2.7.0
  * @access private
  *
- * @staticvar WP_Http $http
- *
  * @return WP_Http HTTP Transport object.
  */
 function _wp_http_get_object() {
@@ -39,9 +37,9 @@ function _wp_http_get_object() {
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_safe_remote_request( $url, $args = array() ) {
 	$args['reject_unsafe_urls'] = true;
@@ -60,9 +58,9 @@ function wp_safe_remote_request( $url, $args = array() ) {
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_safe_remote_get( $url, $args = array() ) {
 	$args['reject_unsafe_urls'] = true;
@@ -81,9 +79,9 @@ function wp_safe_remote_get( $url, $args = array() ) {
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_safe_remote_post( $url, $args = array() ) {
 	$args['reject_unsafe_urls'] = true;
@@ -102,9 +100,9 @@ function wp_safe_remote_post( $url, $args = array() ) {
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url Site URL to retrieve.
- * @param array $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @param string $url  URL to retrieve.
+ * @param array  $args Optional. Request arguments. Default empty array.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_safe_remote_head( $url, $args = array() ) {
 	$args['reject_unsafe_urls'] = true;
@@ -113,40 +111,34 @@ function wp_safe_remote_head( $url, $args = array() ) {
 }
 
 /**
- * Retrieve the raw response from the HTTP request.
+ * Performs an HTTP request and returns its response.
  *
- * The array structure is a little complex:
+ * There are other API functions available which abstract away the HTTP method:
  *
- *     $res = array(
- *         'headers'  => array(),
- *         'response' => array(
- *             'code'    => int,
- *             'message' => string
- *         )
- *     );
- *
- * All of the headers in $res['headers'] are with the name as the key and the
- * value as the value. So to get the User-Agent, you would do the following.
- *
- *     $user_agent = $res['headers']['user-agent'];
- *
- * The body is the raw response content and can be retrieved from $res['body'].
- *
- * This function is called first to make the request and there are other API
- * functions to abstract out the above convoluted setup.
- *
- * Request method defaults for helper functions:
  *  - Default 'GET'  for wp_remote_get()
  *  - Default 'POST' for wp_remote_post()
  *  - Default 'HEAD' for wp_remote_head()
  *
  * @since 2.7.0
  *
- * @see WP_Http::request() For additional information on default arguments.
+ * @see WP_Http::request() For information on default arguments.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error {
+ *     The response array or a WP_Error on failure.
+ *
+ *     @type string[]                       $headers       Array of response headers keyed by their name.
+ *     @type string                         $body          Response body.
+ *     @type array                          $response      {
+ *         Data about the HTTP response.
+ *
+ *         @type int|false    $code    HTTP response code.
+ *         @type string|false $message HTTP response message.
+ *     }
+ *     @type WP_HTTP_Cookie[]               $cookies       Array of response cookies.
+ *     @type WP_HTTP_Requests_Response|null $http_response Raw HTTP response object.
+ * }
  */
 function wp_remote_request( $url, $args = array() ) {
 	$http = _wp_http_get_object();
@@ -154,16 +146,16 @@ function wp_remote_request( $url, $args = array() ) {
 }
 
 /**
- * Retrieve the raw response from the HTTP request using the GET method.
+ * Performs an HTTP request using the GET method and returns its response.
  *
  * @since 2.7.0
  *
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_remote_get( $url, $args = array() ) {
 	$http = _wp_http_get_object();
@@ -171,16 +163,16 @@ function wp_remote_get( $url, $args = array() ) {
 }
 
 /**
- * Retrieve the raw response from the HTTP request using the POST method.
+ * Performs an HTTP request using the POST method and returns its response.
  *
  * @since 2.7.0
  *
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_remote_post( $url, $args = array() ) {
 	$http = _wp_http_get_object();
@@ -188,16 +180,16 @@ function wp_remote_post( $url, $args = array() ) {
 }
 
 /**
- * Retrieve the raw response from the HTTP request using the HEAD method.
+ * Performs an HTTP request using the HEAD method and returns its response.
  *
  * @since 2.7.0
  *
  * @see wp_remote_request() For more information on the response array format.
  * @see WP_Http::request() For default arguments information.
  *
- * @param string $url  Site URL to retrieve.
+ * @param string $url  URL to retrieve.
  * @param array  $args Optional. Request arguments. Default empty array.
- * @return WP_Error|array The response or WP_Error on failure.
+ * @return array|WP_Error The response or WP_Error on failure.
  */
 function wp_remote_head( $url, $args = array() ) {
 	$http = _wp_http_get_object();
@@ -212,8 +204,9 @@ function wp_remote_head( $url, $args = array() ) {
  *
  * @see \Requests_Utility_CaseInsensitiveDictionary
  *
- * @param array $response HTTP response.
- * @return array|\Requests_Utility_CaseInsensitiveDictionary The headers of the response. Empty array if incorrect parameter given.
+ * @param array|WP_Error $response HTTP response.
+ * @return \Requests_Utility_CaseInsensitiveDictionary|array The headers of the response, or empty array
+ *                                                           if incorrect parameter given.
  */
 function wp_remote_retrieve_headers( $response ) {
 	if ( is_wp_error( $response ) || ! isset( $response['headers'] ) ) {
@@ -228,9 +221,10 @@ function wp_remote_retrieve_headers( $response ) {
  *
  * @since 2.7.0
  *
- * @param array  $response
- * @param string $header Header name to retrieve value from.
- * @return string The header value. Empty string on if incorrect parameter given, or if the header doesn't exist.
+ * @param array|WP_Error $response HTTP response.
+ * @param string         $header   Header name to retrieve value from.
+ * @return array|string The header(s) value(s). Array if multiple headers with the same name are retrieved.
+ *                      Empty string if incorrect parameter given, or if the header doesn't exist.
  */
 function wp_remote_retrieve_header( $response, $header ) {
 	if ( is_wp_error( $response ) || ! isset( $response['headers'] ) ) {
@@ -247,12 +241,12 @@ function wp_remote_retrieve_header( $response, $header ) {
 /**
  * Retrieve only the response code from the raw response.
  *
- * Will return an empty array if incorrect parameter value is given.
+ * Will return an empty string if incorrect parameter value is given.
  *
  * @since 2.7.0
  *
- * @param array $response HTTP response.
- * @return int|string The response code as an integer. Empty string on incorrect parameter given.
+ * @param array|WP_Error $response HTTP response.
+ * @return int|string The response code as an integer. Empty string if incorrect parameter given.
  */
 function wp_remote_retrieve_response_code( $response ) {
 	if ( is_wp_error( $response ) || ! isset( $response['response'] ) || ! is_array( $response['response'] ) ) {
@@ -265,12 +259,12 @@ function wp_remote_retrieve_response_code( $response ) {
 /**
  * Retrieve only the response message from the raw response.
  *
- * Will return an empty array if incorrect parameter value is given.
+ * Will return an empty string if incorrect parameter value is given.
  *
  * @since 2.7.0
  *
- * @param array $response HTTP response.
- * @return string The response message. Empty string on incorrect parameter given.
+ * @param array|WP_Error $response HTTP response.
+ * @return string The response message. Empty string if incorrect parameter given.
  */
 function wp_remote_retrieve_response_message( $response ) {
 	if ( is_wp_error( $response ) || ! isset( $response['response'] ) || ! is_array( $response['response'] ) ) {
@@ -285,7 +279,7 @@ function wp_remote_retrieve_response_message( $response ) {
  *
  * @since 2.7.0
  *
- * @param array $response HTTP response.
+ * @param array|WP_Error $response HTTP response.
  * @return string The body of the response. Empty string if no body or incorrect parameter given.
  */
 function wp_remote_retrieve_body( $response ) {
@@ -301,8 +295,9 @@ function wp_remote_retrieve_body( $response ) {
  *
  * @since 4.4.0
  *
- * @param array $response HTTP response.
- * @return array An array of `WP_Http_Cookie` objects from the response. Empty array if there are none, or the response is a WP_Error.
+ * @param array|WP_Error $response HTTP response.
+ * @return WP_Http_Cookie[] An array of `WP_Http_Cookie` objects from the response.
+ *                          Empty array if there are none, or the response is a WP_Error.
  */
 function wp_remote_retrieve_cookies( $response ) {
 	if ( is_wp_error( $response ) || empty( $response['cookies'] ) ) {
@@ -317,9 +312,10 @@ function wp_remote_retrieve_cookies( $response ) {
  *
  * @since 4.4.0
  *
- * @param array  $response HTTP response.
- * @param string $name     The name of the cookie to retrieve.
- * @return WP_Http_Cookie|string The `WP_Http_Cookie` object. Empty string if the cookie isn't present in the response.
+ * @param array|WP_Error $response HTTP response.
+ * @param string         $name     The name of the cookie to retrieve.
+ * @return WP_Http_Cookie|string The `WP_Http_Cookie` object, or empty string
+ *                               if the cookie is not present in the response.
  */
 function wp_remote_retrieve_cookie( $response, $name ) {
 	$cookies = wp_remote_retrieve_cookies( $response );
@@ -342,9 +338,10 @@ function wp_remote_retrieve_cookie( $response, $name ) {
  *
  * @since 4.4.0
  *
- * @param array  $response HTTP response.
- * @param string $name     The name of the cookie to retrieve.
- * @return string The value of the cookie. Empty string if the cookie isn't present in the response.
+ * @param array|WP_Error $response HTTP response.
+ * @param string         $name     The name of the cookie to retrieve.
+ * @return string The value of the cookie, or empty string
+ *                if the cookie is not present in the response.
  */
 function wp_remote_retrieve_cookie_value( $response, $name ) {
 	$cookie = wp_remote_retrieve_cookie( $response, $name );
@@ -374,14 +371,14 @@ function wp_http_supports( $capabilities = array(), $url = null ) {
 
 	$count = count( $capabilities );
 
-	// If we have a numeric $capabilities array, spoof a wp_remote_request() associative $args array
+	// If we have a numeric $capabilities array, spoof a wp_remote_request() associative $args array.
 	if ( $count && count( array_filter( array_keys( $capabilities ), 'is_numeric' ) ) == $count ) {
 		$capabilities = array_combine( array_values( $capabilities ), array_fill( 0, $count, true ) );
 	}
 
 	if ( $url && ! isset( $capabilities['ssl'] ) ) {
 		$scheme = parse_url( $url, PHP_URL_SCHEME );
-		if ( 'https' == $scheme || 'ssl' == $scheme ) {
+		if ( 'https' === $scheme || 'ssl' === $scheme ) {
 			$capabilities['ssl'] = true;
 		}
 	}
@@ -417,13 +414,13 @@ function get_http_origin() {
  *
  * @since 3.4.0
  *
- * @return array Array of origin URLs.
+ * @return string[] Array of origin URLs.
  */
 function get_allowed_http_origins() {
 	$admin_origin = parse_url( admin_url() );
 	$home_origin  = parse_url( home_url() );
 
-	// @todo preserve port?
+	// @todo Preserve port?
 	$allowed_origins = array_unique(
 		array(
 			'http://' . $admin_origin['host'],
@@ -438,12 +435,13 @@ function get_allowed_http_origins() {
 	 *
 	 * @since 3.4.0
 	 *
-	 * @param array $allowed_origins {
-	 *     Default allowed HTTP origins.
-	 *     @type string Non-secure URL for admin origin.
-	 *     @type string Secure URL for admin origin.
-	 *     @type string Non-secure URL for home origin.
-	 *     @type string Secure URL for home origin.
+	 * @param string[] $allowed_origins {
+	 *     Array of default allowed HTTP origins.
+	 *
+	 *     @type string $0 Non-secure URL for admin origin.
+	 *     @type string $1 Secure URL for admin origin.
+	 *     @type string $2 Non-secure URL for home origin.
+	 *     @type string $3 Secure URL for home origin.
 	 * }
 	 */
 	return apply_filters( 'allowed_http_origins', $allowed_origins );
@@ -454,7 +452,7 @@ function get_allowed_http_origins() {
  *
  * @since 3.4.0
  *
- * @param null|string $origin Origin URL. If not provided, the value of get_http_origin() is used.
+ * @param string|null $origin Origin URL. If not provided, the value of get_http_origin() is used.
  * @return string Origin URL if allowed, empty string if not.
  */
 function is_allowed_http_origin( $origin = null ) {
@@ -464,7 +462,7 @@ function is_allowed_http_origin( $origin = null ) {
 		$origin = get_http_origin();
 	}
 
-	if ( $origin && ! in_array( $origin, get_allowed_http_origins() ) ) {
+	if ( $origin && ! in_array( $origin, get_allowed_http_origins(), true ) ) {
 		$origin = '';
 	}
 
@@ -496,8 +494,8 @@ function send_origin_headers() {
 	$origin = get_http_origin();
 
 	if ( is_allowed_http_origin( $origin ) ) {
-		@header( 'Access-Control-Allow-Origin: ' . $origin );
-		@header( 'Access-Control-Allow-Credentials: true' );
+		header( 'Access-Control-Allow-Origin: ' . $origin );
+		header( 'Access-Control-Allow-Credentials: true' );
 		if ( 'OPTIONS' === $_SERVER['REQUEST_METHOD'] ) {
 			exit;
 		}
@@ -517,17 +515,21 @@ function send_origin_headers() {
  *
  * @since 3.5.2
  *
- * @param string $url
- * @return false|string URL or false on failure.
+ * @param string $url Request URL.
+ * @return string|false URL or false on failure.
  */
 function wp_http_validate_url( $url ) {
+	if ( ! is_string( $url ) || '' === $url || is_numeric( $url ) ) {
+		return false;
+	}
+
 	$original_url = $url;
 	$url          = wp_kses_bad_protocol( $url, array( 'http', 'https' ) );
 	if ( ! $url || strtolower( $url ) !== strtolower( $original_url ) ) {
 		return false;
 	}
 
-	$parsed_url = @parse_url( $url );
+	$parsed_url = parse_url( $url );
 	if ( ! $parsed_url || empty( $parsed_url['host'] ) ) {
 		return false;
 	}
@@ -540,22 +542,17 @@ function wp_http_validate_url( $url ) {
 		return false;
 	}
 
-	$parsed_home = @parse_url( get_option( 'home' ) );
-
-	if ( isset( $parsed_home['host'] ) ) {
-		$same_host = strtolower( $parsed_home['host'] ) === strtolower( $parsed_url['host'] );
-	} else {
-		$same_host = false;
-	}
+	$parsed_home = parse_url( get_option( 'home' ) );
+	$same_host   = isset( $parsed_home['host'] ) && strtolower( $parsed_home['host'] ) === strtolower( $parsed_url['host'] );
+	$host        = trim( $parsed_url['host'], '.' );
 
 	if ( ! $same_host ) {
-		$host = trim( $parsed_url['host'], '.' );
 		if ( preg_match( '#^(([1-9]?\d|1\d\d|25[0-5]|2[0-4]\d)\.){3}([1-9]?\d|1\d\d|25[0-5]|2[0-4]\d)$#', $host ) ) {
 			$ip = $host;
 		} else {
 			$ip = gethostbyname( $host );
-			if ( $ip === $host ) { // Error condition for gethostbyname()
-				$ip = false;
+			if ( $ip === $host ) { // Error condition for gethostbyname().
+				return false;
 			}
 		}
 		if ( $ip ) {
@@ -572,9 +569,9 @@ function wp_http_validate_url( $url ) {
 				 *
 				 * @since 3.6.0
 				 *
-				 * @param bool   false Whether HTTP request is external or not.
-				 * @param string $host IP of the requested host.
-				 * @param string $url  URL of the requested host.
+				 * @param bool   $external Whether HTTP request is external or not.
+				 * @param string $host     Host name of the requested URL.
+				 * @param string $url      Requested URL.
 				 */
 				if ( ! apply_filters( 'http_request_host_is_external', false, $host, $url ) ) {
 					return false;
@@ -588,7 +585,20 @@ function wp_http_validate_url( $url ) {
 	}
 
 	$port = $parsed_url['port'];
-	if ( 80 === $port || 443 === $port || 8080 === $port ) {
+
+	/**
+	 * Controls the list of ports considered safe in HTTP API.
+	 *
+	 * Allows to change and allow external requests for the HTTP request.
+	 *
+	 * @since 5.9.0
+	 *
+	 * @param array  $allowed_ports Array of integers for valid ports.
+	 * @param string $host          Host name of the requested URL.
+	 * @param string $url           Requested URL.
+	 */
+	$allowed_ports = apply_filters( 'http_allowed_safe_ports', array( 80, 443, 8080 ), $host, $url );
+	if ( is_array( $allowed_ports ) && in_array( $port, $allowed_ports, true ) ) {
 		return $url;
 	}
 
@@ -600,7 +610,7 @@ function wp_http_validate_url( $url ) {
 }
 
 /**
- * Whitelists allowed redirect hosts for safe HTTP requests as well.
+ * Mark allowed redirect hosts safe for HTTP requests as well.
  *
  * Attached to the {@see 'http_request_host_is_external'} filter.
  *
@@ -618,14 +628,14 @@ function allowed_http_request_hosts( $is_external, $host ) {
 }
 
 /**
- * Whitelists any domain in a multisite installation for safe HTTP requests.
+ * Adds any domain in a multisite installation for safe HTTP requests to the
+ * allowed list.
  *
  * Attached to the {@see 'http_request_host_is_external'} filter.
  *
  * @since 3.6.0
  *
  * @global wpdb $wpdb WordPress database abstraction object.
- * @staticvar array $queried
  *
  * @param bool   $is_external
  * @param string $host
@@ -637,7 +647,7 @@ function ms_allowed_http_request_hosts( $is_external, $host ) {
 	if ( $is_external ) {
 		return $is_external;
 	}
-	if ( $host === get_network()->domain ) {
+	if ( get_network()->domain === $host ) {
 		return true;
 	}
 	if ( isset( $queried[ $host ] ) ) {
@@ -648,24 +658,20 @@ function ms_allowed_http_request_hosts( $is_external, $host ) {
 }
 
 /**
- * A wrapper for PHP's parse_url() function that handles consistency in the return
- * values across PHP versions.
+ * A wrapper for PHP's parse_url() function that handles consistency in the return values
+ * across PHP versions.
  *
- * PHP 5.4.7 expanded parse_url()'s ability to handle non-absolute url's, including
- * schemeless and relative url's with :// in the path. This function works around
+ * PHP 5.4.7 expanded parse_url()'s ability to handle non-absolute URLs, including
+ * schemeless and relative URLs with "://" in the path. This function works around
  * those limitations providing a standard output on PHP 5.2~5.4+.
  *
- * Secondly, across various PHP versions, schemeless URLs starting containing a ":"
- * in the query are being handled inconsistently. This function works around those
- * differences as well.
- *
- * Error suppression is used as prior to PHP 5.3.3, an E_WARNING would be generated
- * when URL parsing failed.
+ * Secondly, across various PHP versions, schemeless URLs containing a ":" in the query
+ * are being handled inconsistently. This function works around those differences as well.
  *
  * @since 4.4.0
  * @since 4.7.0 The `$component` parameter was added for parity with PHP's `parse_url()`.
  *
- * @link https://secure.php.net/manual/en/function.parse-url.php
+ * @link https://www.php.net/manual/en/function.parse-url.php
  *
  * @param string $url       The URL to parse.
  * @param int    $component The specific component to retrieve. Use one of the PHP
@@ -678,7 +684,7 @@ function ms_allowed_http_request_hosts( $is_external, $host ) {
  */
 function wp_parse_url( $url, $component = -1 ) {
 	$to_unset = array();
-	$url      = strval( $url );
+	$url      = (string) $url;
 
 	if ( '//' === substr( $url, 0, 2 ) ) {
 		$to_unset[] = 'scheme';
@@ -689,7 +695,7 @@ function wp_parse_url( $url, $component = -1 ) {
 		$url        = 'placeholder://placeholder' . $url;
 	}
 
-	$parts = @parse_url( $url );
+	$parts = parse_url( $url );
 
 	if ( false === $parts ) {
 		// Parsing failure.
@@ -712,12 +718,12 @@ function wp_parse_url( $url, $component = -1 ) {
  * @since 4.7.0
  * @access private
  *
- * @link https://secure.php.net/manual/en/function.parse-url.php
+ * @link https://www.php.net/manual/en/function.parse-url.php
  *
  * @param array|false $url_parts The parsed URL. Can be false if the URL failed to parse.
- * @param int    $component The specific component to retrieve. Use one of the PHP
- *                          predefined constants to specify which one.
- *                          Defaults to -1 (= return all parts as an array).
+ * @param int         $component The specific component to retrieve. Use one of the PHP
+ *                               predefined constants to specify which one.
+ *                               Defaults to -1 (= return all parts as an array).
  * @return mixed False on parse failure; Array of URL components on success;
  *               When a specific component has been requested: null if the component
  *               doesn't exist in the given URL; a string or - in the case of
@@ -744,10 +750,10 @@ function _get_component_from_parsed_url_array( $url_parts, $component = -1 ) {
  * @since 4.7.0
  * @access private
  *
- * @link https://secure.php.net/manual/en/url.constants.php
+ * @link https://www.php.net/manual/en/url.constants.php
  *
  * @param int $constant PHP_URL_* constant.
- * @return string|bool The named key or false.
+ * @return string|false The named key or false.
  */
 function _wp_translate_php_url_constant_to_key( $constant ) {
 	$translation = array(
